@@ -19,9 +19,9 @@
 
 """Provides the FocusTrackingPresenter for Orca."""
 
-__id__        = "$Id: focus_tracking_presenter.py,v 1.95 2006/12/13 22:35:06 wwalker Exp $"
-__version__   = "$Revision: 1.95 $"
-__date__      = "$Date: 2006/12/13 22:35:06 $"
+__id__        = "$Id: focus_tracking_presenter.py 1996 2007-02-11 18:07:28Z joanied $"
+__version__   = "$Revision: 1996 $"
+__date__      = "$Date: 2007-02-11 13:07:28 -0500 (Sun, 11 Feb 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -453,7 +453,15 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         #
         retryCount = 0
         oldLocusOfFocus = orca_state.locusOfFocus
-        while retryCount <= settings.commFailureAttemptLimit:
+        try:
+            s = self._getScript(event.source.app)
+        except:
+            s = None
+            debug.printException(debug.LEVEL_WARNING)
+            debug.println(debug.LEVEL_WARNING,
+                          "ERROR: received an event, but Script is None")
+
+        while s and retryCount <= s.commFailureAttemptLimit:
             try:
                 if not event.source.state.count( \
                                       atspi.Accessibility.STATE_ICONIFIED):
@@ -496,7 +504,6 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                         braille.setupKeyRanges(\
                             orca_state.activeScript.brailleBindings.keys())
 
-                    s = self._getScript(event.source.app)
                     s.processObjectEvent(event)
                     if retryCount:
                         debug.println(debug.LEVEL_WARNING,
@@ -508,7 +515,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                               "COMM_FAILURE above while processing: " \
                               + event.type)
                 retryCount += 1
-                if retryCount <= settings.commFailureAttemptLimit:
+                if retryCount <= s.commFailureAttemptLimit:
                     # We want the old locus of focus to be reset so
                     # the proper stuff will be spoken if the locus
                     # of focus changed during our last attempt at
@@ -517,7 +524,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                     orca_state.locusOfFocus = oldLocusOfFocus
                     debug.println(debug.LEVEL_WARNING,
                                   "  TRYING AGAIN (%d)" % retryCount)
-                    time.sleep(settings.commFailureWaitTime)
+                    time.sleep(s.commFailureWaitTime)
                 else:
                     debug.println(debug.LEVEL_WARNING,
                                   "  GIVING UP AFTER %d TRIES" \
@@ -601,9 +608,18 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             # we want it to ref, thus allowing things to survive until
             # they are processed on the gidle thread.
             #
+            # If the event doesn't have a source or that source is not marked
+            # valid, then we don't care about this event. Just return.
+            #
+            event = atspi.Event(e)
+            if not event.source or not event.source.valid:
+                debug.println(debug.LEVEL_FINEST,
+                      "---------> IGNORING INVALID EVENT %s" % e.type)
+                if settings.debugEventQueue:
+                    self._enqueueEventCount -= 1
+                return
             debug.println(debug.LEVEL_FINEST,
                           "---------> QUEUEING EVENT %s" % e.type)
-            event = atspi.Event(e)
 
         if event:
             if settings.debugEventQueue:
