@@ -19,9 +19,9 @@
 
 """The main module for the Orca screen reader."""
 
-__id__        = "$Id: orca.py,v 1.171 2006/11/30 16:38:58 richb Exp $"
-__version__   = "$Revision: 1.171 $"
-__date__      = "$Date: 2006/11/30 16:38:58 $"
+__id__        = "$Id: orca.py 1818 2006-12-13 01:21:31Z wwalker $"
+__version__   = "$Revision: 1818 $"
+__date__      = "$Date: 2006-12-12 20:21:31 -0500 (Tue, 12 Dec 2006) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -646,6 +646,25 @@ def _keyEcho(event):
 
         speech.speak(event_string, voice)
 
+def _processKeyCaptured(event):
+    """Called when a new key event arrives and orca_state.capturingKeys=True.
+    (used for key bindings redefinition)
+    """
+
+    if event.type == 0:
+        if _isModifierKey(event.event_string) \
+               or event.event_string == "Return":
+            pass
+        elif event.event_string == "Escape":
+            orca_state.capturingKeys = False
+        else:
+            speech.speak(_("Key captured: %s. Press enter to confirm.") \
+                         % str(event.event_string))
+            orca_state.lastCapturedKey = event
+    else:
+        pass
+    return False
+
 def _processKeyboardEvent(event):
     """The primary key event handler for Orca.  Keeps track of various
     attributes, such as the lastInputEvent.  Also calls keyEcho as well
@@ -713,26 +732,29 @@ def _processKeyboardEvent(event):
     #
     consumed = False
     try:
-        consumed = _keyBindings.consumeKeyboardEvent(None, keyboardEvent)
-        if (not consumed) and (_currentPresentationManager >= 0):
-            consumed = _PRESENTATION_MANAGERS[_currentPresentationManager].\
-                       processKeyboardEvent(keyboardEvent)
-        if (not consumed) and settings.learnModeEnabled:
-            if keyboardEvent.type \
-                == atspi.Accessibility.KEY_PRESSED_EVENT:
-                clickCount = util.getClickCount(orca_state.lastInputEvent,
-                                            keyboardEvent)
-                if clickCount == 2:
-                    util.phoneticSpellCurrentItem(keyboardEvent.event_string)
-                else:
-                    # Check to see if there are localized words to be
-                    # spoken for this key event.
-                    #
-                    braille.displayMessage(keyboardEvent.event_string)
-                    event_string = keyboardEvent.event_string
-                    event_string = keynames.getKeyName(event_string)
-                    speech.speak(event_string)
-            consumed = True
+        if orca_state.capturingKeys:
+            _processKeyCaptured(keyboardEvent)
+        else:
+            consumed = _keyBindings.consumeKeyboardEvent(None, keyboardEvent)
+            if (not consumed) and (_currentPresentationManager >= 0):
+                consumed = _PRESENTATION_MANAGERS[_currentPresentationManager].\
+                           processKeyboardEvent(keyboardEvent)
+            if (not consumed) and settings.learnModeEnabled:
+                if keyboardEvent.type \
+                    == atspi.Accessibility.KEY_PRESSED_EVENT:
+                    clickCount = util.getClickCount(orca_state.lastInputEvent,
+                                                keyboardEvent)
+                    if clickCount == 2:
+                        util.phoneticSpellCurrentItem(keyboardEvent.event_string)
+                    else:
+                        # Check to see if there are localized words to be
+                        # spoken for this key event.
+                        #
+                        braille.displayMessage(keyboardEvent.event_string)
+                        event_string = keyboardEvent.event_string
+                        event_string = keynames.getKeyName(event_string)
+                        speech.speak(event_string)
+                consumed = True
     except:
         debug.printException(debug.LEVEL_SEVERE)
 
@@ -854,7 +876,7 @@ def loadUserSettings(script=None, inputEvent=None):
 
     if settings.enableBraille:
         try:
-            braille.init(_processBrailleEvent, 7)
+            braille.init(_processBrailleEvent, settings.tty)
         except:
             debug.printException(debug.LEVEL_WARNING)
             debug.println(debug.LEVEL_WARNING,
@@ -1192,8 +1214,8 @@ def usage():
     print _("-t, --text-setup             Set up user preferences (text version)")
     print _("-n, --no-setup               Skip set up of user preferences")
     print _("-u, --user-prefs-dir=dirname Use alternate directory for user preferences")
-    print _("-e, --enable=[speech|braille|braille-monitor|magnifier] Force use of option")
-    print _("-d, --disable=[speech|braille|braille-monitor|magnifier] Prevent use of option")
+    print _("-e, --enable=[speech|braille|braille-monitor|magnifier|main-window] Force use of option")
+    print _("-d, --disable=[speech|braille|braille-monitor|magnifier|main-window] Prevent use of option")
     print _("-q, --quit                   Quits Orca (if shell script used)")
     print
     print _("If Orca has not been previously set up by the user, Orca\nwill automatically launch the preferences set up unless\nthe -n or --no-setup option is used.")
@@ -1295,6 +1317,8 @@ def main():
                     _commandLineSettings["enableBrailleMonitor"] = True
                 elif feature == "magnifier":
                     _commandLineSettings["enableMagnifier"] = True
+                elif feature == "main-window":
+                    _commandLineSettings["showMainWindow"] = True
                 else:
                     usage()
                     os._exit(2)
@@ -1309,6 +1333,8 @@ def main():
                     _commandLineSettings["enableBrailleMonitor"] = False
                 elif feature == "magnifier":
                     _commandLineSettings["enableMagnifier"] = False
+                elif feature == "main-window":
+                    _commandLineSettings["showMainWindow"] = False
                 else:
                     usage()
                     os._exit(2)
@@ -1333,11 +1359,16 @@ def main():
         os._exit(2)
 
     # Do not run Orca if accessibility has not been enabled.
+    # We do allow, however, one to force Orca to run via the
+    # "-n" switch.  The main reason is so that things such
+    # as accessible login can work -- in those cases, the gconf
+    # setting is typically not set since the gdm user does not
+    # have a home.
     #
     import commands
     a11yEnabled = commands.getoutput(\
         "gconftool-2 --get /desktop/gnome/interface/accessibility")
-    if a11yEnabled != "true":
+    if (not bypassSetup) and (a11yEnabled != "true"):
         _showPreferencesConsole()
         abort()
 
